/*
		Private Chat [Room], v1.1, by Alka


 o Description

	Like title says, with this plugin you can have a private conversation with players.
	You can also create private chat rooms so you can talk with more players at the same time.
	Invitations to private room can be made by room owner ofc.
	When you try to invite someone to a private [room] conversation, 
	a menu with confirmation will appear to that player so he can choose YES/NO.
	Use colored chat messages.

 o Commands (say chat only)

	/privatehelp - displays private chat help
	/private [name|#userid|authid] - starts a private conversation with specified player
	/unprivate - leaves the conversation with actual player
	/private room [number] - creates a private chat room with specified room number
	/unprivate room - leaves the private chat room, including the chaters if you are the owner
	/invite [name|#userid|authid] - invites specified player to your private chat room

 o Changelog

    v1.1 - Few fixes and added /privatehelp option
    v1.0 - Initial release

*/

// Admin access flag(s)
#define FLAG_ACCESS ADMIN_ALL

#include <translator>
#include <amxmod>
#include <amxmisc>

#define CHAT_LEN_MAX 106
#define NO_PRIVACY 0
#define NO_ROOM -1

#define MAX_CHAT_ROOMS 5
#define MAX_PLAYERS 32

new g_iRoomOwner[MAX_CHAT_ROOMS + 1];
new g_iPrivateChat[MAX_PLAYERS + 1];
new g_iPrivateRoom[MAX_PLAYERS + 1];
new g_iInviter[MAX_PLAYERS + 1];

new g_iMaxPlayers;
new g_iMsgIDSayText;

public plugin_init() {
	load_translations("private_chat");
	
	register_plugin(_T("Private Chat [Room]"), "1.1", "Alka");
	
	register_clcmd("say", "clcmdHandleSay", FLAG_ACCESS, _T("/privatehelp - displays private chat help"));
	register_clcmd("say_team", "clcmdHandleSayTeam", -1);
	
	register_menucmd(register_menuid("Invite"), 1<<0|1<<1, "InviteMenu_Handle");
	
	g_iMaxPlayers = get_maxplayers();
	g_iMsgIDSayText = get_user_msgid("SayText");
}

public client_connect(id)
{
	g_iPrivateChat[id] = NO_PRIVACY;
	g_iPrivateRoom[id] = NO_PRIVACY;
}

public client_disconnect(id)
{
	new iIsRoomOwner = CheckRoomOwner(id);
	new iPlayer;
	
	if(iIsRoomOwner != NO_ROOM)
	{
		ResetChaters(iIsRoomOwner);
		g_iRoomOwner[iIsRoomOwner] = 0;
	}
	else if((iPlayer = g_iPrivateRoom[id]) > 0)
	{
		new szName[32];
		get_user_name(id, szName, charsmax(szName));
		
		for(new i = 1 ; i <= g_iMaxPlayers ; i++)
		{
			if(i == id || g_iPrivateRoom[i] != iPlayer)
				continue;
			
			client_print_color2(id, i, _T("(^4ROOM %i^1) ^3%s ^4is leaving the chat room (^1player disconnected^4)."), g_iPrivateRoom[i], szName);
		}
		g_iPrivateRoom[id] = NO_PRIVACY;
	}
	else if((iPlayer = g_iPrivateChat[id]) > 0)
	{
		if(g_iPrivateChat[iPlayer] != NO_PRIVACY)
		{
			new szName[32];
			get_user_name(id, szName, charsmax(szName));
			
			g_iPrivateChat[id] = NO_PRIVACY;
			g_iPrivateChat[iPlayer] = NO_PRIVACY;
			
			client_print_color2(id, iPlayer, _T("^4You have left the private conversation with ^3%s^4 (^1player disconnected^4)."), szName);
		}
	}
}

public clcmdHandleSay(id, iLevel)
{
	if(!access(id, iLevel))
	{
		return PLUGIN_CONTINUE;
	}
	
	new szArg[3];
	read_argv(1, szArg, charsmax(szArg));
	
	if(szArg[0] != '/' || (szArg[1] != 'p' && szArg[1] != 'P' && szArg[1] != 'u' && szArg[1] != 'U' && szArg[1] != 'i' && szArg[1] != 'I'))
		return PLUGIN_CONTINUE;
	
	static szArgs[CHAT_LEN_MAX];
	new iArgsLen = read_args(szArgs, charsmax(szArgs));
	new bool:bContainQuotes = bool:(szArgs[0] == '"' && szArgs[iArgsLen - 1] == '"');
	trim_imp(szArgs);
	
	new szCommand[16], szRoom[4];
	parse(szArgs, szCommand, charsmax(szCommand), szArgs, charsmax(szArgs), szRoom, charsmax(szRoom));
	
	if(equal(szCommand, "/privatehelp"))
	{
		if(bContainQuotes == true)
		{
			client_print_color2(id, id, _T("^4Open now your console to see the private chat help."));
		}
		
		console_print(id, _T("Usage: say </privatehelp|/private|/unprivate|/invite> [...]^nCommands:"));
		console_print(id, _T("    /privatehelp - displays this help"));
		console_print(id, _T("    /private [name|#userid|authid] - starts a private conversation with specified player"));
		console_print(id, _T("    /unprivate - leaves the conversation with actual player"));
		console_print(id, _T("    /private room [number] - creates a private chat room with specified room number"));
		console_print(id, _T("    /unprivate room - leaves the private chat room, including the chaters if you are the owner"));
		console_print(id, _T("    /invite [name|#userid|authid] - invites specified player to your private chat room"));
		return PLUGIN_HANDLED;
	}
	else if(equal(szCommand, "/private"))
	{
		if(equal(szArgs, "room"))
		{
			new iIsRoomOwner = CheckRoomOwner(id);
			if(iIsRoomOwner != NO_ROOM)
			{
				client_print_color2(id, id, _T("^4You can't start private chat room because you are already the owner of the chat room ^1#%d^4. Leave it in first!"), iIsRoomOwner);
				return PLUGIN_HANDLED;
			}
			
			new iRoom = str_to_num(szRoom);
			if(!(1 <= iRoom <= MAX_CHAT_ROOMS))
			{
				client_print_color2(id, id, _T("^4Invalid room number."));
				return PLUGIN_HANDLED;
			}
			
			if(g_iRoomOwner[iRoom] > 0 && g_iRoomOwner[iRoom] != id)
			{
				client_print_color2(id, id, _T("^4Someone else is the owner of this chat room. To enter, ask for invitaion."));
			}
			else if(!g_iRoomOwner[iRoom] && g_iPrivateChat[id] == NO_PRIVACY)
			{
				g_iPrivateRoom[id] = iRoom;
				g_iRoomOwner[iRoom] = id;
				
				client_print_color2(id, id, _T("^4You are entering private chat room ^1#%d^4. To invite someone type ^"^1/invite <name>^4^"."), iRoom);
			}
		}
		else
		{
			if(g_iPrivateChat[id] != NO_PRIVACY || g_iPrivateRoom[id] != NO_PRIVACY)
			{
				client_print_color2(id, id, _T("^4You are already in a private [room] conversation."));
				return PLUGIN_HANDLED;
			}
			
			new iPlayer = cmd_target_private_chat(id, szArgs, 8, bContainQuotes);
			if(iPlayer)
			{
				if(g_iPrivateChat[iPlayer] != NO_PRIVACY || g_iPrivateRoom[iPlayer] != NO_PRIVACY)
				{
					new szPlName[32];
					get_user_name(iPlayer, szPlName, charsmax(szPlName));
					
					client_print_color2(iPlayer, id, _T("^3%s ^4is already in a private [room] conversation."), szPlName);
				}
				else
				{
					RequestInvitation(id, iPlayer, 0);
				}
			}
		}
		return PLUGIN_HANDLED
	}
	else if(equal(szCommand, "/unprivate"))
	{
		if(equal(szArgs, "room"))
		{
			if(!g_iPrivateRoom[id])
			{
				client_print_color2(id, id, _T("^4You are not in any private chat room."));
			}
			else
			{
				new iIsRoomOwner = CheckRoomOwner(id);
				
				if(iIsRoomOwner != NO_ROOM && g_iPrivateRoom[id] == iIsRoomOwner)
				{
					ResetChaters(iIsRoomOwner);
					
					g_iRoomOwner[iIsRoomOwner] = 0;
					g_iPrivateRoom[id] = NO_PRIVACY;
				}
				else
				{
					new szName[32];
					get_user_name(id, szName, charsmax(szName));
					
					for(new i = 1 ; i <= g_iMaxPlayers ; i++)
					{
						if(i == id || g_iPrivateRoom[i] != g_iPrivateRoom[id])
							continue;
						
						client_print_color2(id, i, _T("(^4ROOM %i^1) ^3%s ^4is leaving the chat room."), g_iPrivateRoom[i], szName);
					}
					client_print_color2(id, id, _T("^4You have left the private chat room ^1#%d^4."), g_iPrivateRoom[id]);
					g_iPrivateRoom[id] = NO_PRIVACY;
				}
			}
		}
		else
		{
			new iPlayer = g_iPrivateChat[id];
			
			if(!iPlayer)
			{
				client_print_color2(id, id, _T("^4You are not in any private chat."));
			}
			else
			{
				if(g_iPrivateChat[iPlayer] != NO_PRIVACY)
				{
					new szName[32];
					get_user_name(id, szName, charsmax(szName));
					
					new szPlName[32];
					get_user_name(iPlayer, szPlName, charsmax(szPlName));
					
					g_iPrivateChat[id] = NO_PRIVACY;
					g_iPrivateChat[iPlayer] = NO_PRIVACY;
					
					client_print_color2(iPlayer, id, _T("^4You have left the private conversation with ^3%s^4."), szPlName);
					client_print_color2(id, iPlayer, _T("^4You have left the private conversation with ^3%s^4."), szName);
				}
			}
		}
		return PLUGIN_HANDLED
	}
	else if(equali(szCommand, "/invite"))
	{
		if(g_iRoomOwner[g_iPrivateRoom[id]] != id)
		{
			client_print_color2(id, id, _T("^4You can't invite someone if your not the owner of chat room."));
		}
		else
		{
			new iPlayer = cmd_target_private_chat(id, szArgs, 8, bContainQuotes);
			if(!iPlayer)
			{
				return PLUGIN_HANDLED;
			}
			
			if(g_iPrivateRoom[iPlayer] != NO_PRIVACY || g_iPrivateChat[iPlayer] != NO_PRIVACY)
			{
				new szPlName[32];
				get_user_name(iPlayer, szPlName, charsmax(szPlName));
				
				client_print_color2(iPlayer, id, _T("^3%s ^4is already in a private [room] conversation."), szPlName);
				return PLUGIN_HANDLED;
			}
			RequestInvitation(id, iPlayer, g_iPrivateRoom[id]);
		}
		return PLUGIN_HANDLED
	}
	return PLUGIN_CONTINUE;
}

public clcmdHandleSayTeam(id)
{
	if(g_iPrivateChat[id] == NO_PRIVACY && g_iPrivateRoom[id] == NO_PRIVACY)
		return PLUGIN_CONTINUE;
	
	new szArgs[CHAT_LEN_MAX];
	read_args(szArgs, charsmax(szArgs));
	
	if(trim_imp(szArgs) == 0)
		return PLUGIN_CONTINUE;
	
	if(g_iPrivateChat[id] != NO_PRIVACY)
	{
		new szName[32];
		get_user_name(id, szName, charsmax(szName));
		
		client_print_color2(id, id, _T("(^4PRIVATE^1) ^3%s ^1:  %s"), szName, szArgs);
		client_print_color2(id, g_iPrivateChat[id], _T("(^4PRIVATE^1) ^3%s ^1:  %s"), szName, szArgs);
		
		return PLUGIN_HANDLED;
	}
	
	if(g_iPrivateRoom[id] != NO_PRIVACY)
	{
		new szName[32];
		get_user_name(id, szName, charsmax(szName));
		
		for(new i = 1 ; i <= g_iMaxPlayers ; i++)
		{
			if(g_iPrivateRoom[i] != g_iPrivateRoom[id])
				continue;
			
			client_print_color2(id, i, _T("(^4ROOM %i^1) ^3%s ^1:  %s"), g_iPrivateRoom[id], szName, szArgs);
		}
		return PLUGIN_HANDLED;
	}
	return PLUGIN_CONTINUE;
}

public InviteMenu_Handle(id, key)
{
	new iInviter = g_iInviter[id];
	
	if(!is_user_connected(iInviter))
		return PLUGIN_HANDLED;
	
	new szName[2][32];
	
	switch(key)
	{
		case 0:
		{
			if(!g_iPrivateRoom[iInviter])
			{
				g_iPrivateChat[id] = iInviter;
				g_iPrivateChat[iInviter] = id;
				
				get_user_name(iInviter, szName[0], charsmax(szName[]));
				get_user_name(id, szName[1], charsmax(szName[]));
				
				client_print_color2(id, iInviter, _T("^4You are entering in a private conversation with ^3%s^4."), szName[1]);
				client_print_color2(iInviter, id, _T("^4You are entering in a private conversation with ^3%s^4."), szName[0]);
				client_print_color2(id, id, _T("^4Use your say_team key, usually ^"^1u^4^" to chat."));
			}
			else
			{
				g_iPrivateRoom[id] = g_iPrivateRoom[iInviter];
				
				get_user_name(id, szName[1], charsmax(szName[])); // fix here
				
				for(new i = 1 ; i <= g_iMaxPlayers ; i++)
				{
					if(i == id || g_iPrivateRoom[i] != g_iPrivateRoom[id])
						continue;
					
					client_print_color2(id, i, _T("(^4ROOM %i^1) ^3%s ^1is entering the chat room."), g_iPrivateRoom[i], szName[1]);
				}
				client_print_color2(id, id, _T("^4You are entering private chat room ^1#%d ^4by invitation."), g_iPrivateRoom[id]);
				client_print_color2(id, id, _T("^4Use your say_team key, usually ^"^1u^4^" to chat."));
			}
		}
		case 1:
		{
			new szName[32];
			get_user_name(id, szName, charsmax(szName));
			
			client_print_color2(id, iInviter, _T("^3%s ^4has denied your invitation to private chat."), szName);
		}
	}
	return PLUGIN_HANDLED;
}

CheckRoomOwner(id)
{
	for(new i = 1; i <= MAX_CHAT_ROOMS; i++)
	{
		if(g_iRoomOwner[i] == id)
			return i;
	}
	return NO_ROOM;
}

ResetChaters(iRoom)
{
	for(new i = 1 ; i <= g_iMaxPlayers ; i++)
	{
		if(g_iPrivateRoom[i] != iRoom)
			continue;
		
		g_iPrivateRoom[i] = NO_PRIVACY;
		client_print_color2(i, i, _T("^4The owner of chat room has left. You are no longer in this chat room."));
	}
}

RequestInvitation(inviter, target, room = 0)
{
	new szMenu[128], iLen;
	new szName[32];
	get_user_name(inviter, szName, charsmax(szName));
	
	if(!room)
	{
		iLen = formatex(szMenu, charsmax(szMenu), _T("\yDo you accept a private conversation from \w%s\y?^n^n", target), szName);
	}
	else
	{
		iLen = formatex(szMenu, charsmax(szMenu), _T("\yDo you accept invitation to chat room \w%d from %s\y?^n^n", target), g_iPrivateRoom[inviter], szName);
	}
	iLen += copy(szMenu[iLen], charsmax(szMenu) - iLen, _T("\w1. Yes^n\w2. No^n", target));
	show_menu(target, 1<<0|1<<1, szMenu, -1, "Invite");
	
	g_iInviter[target] = inviter;
}

/* Finds a player matching arg and flags parameters (IP support + chat color).
 * Flags:
 *  1 - obey immunity
 *  2 - allow yourself
 *  4 - must be alive
 *  8 - can't be a bot
 * Note: If you choose do not put any flag, immunity and yourself flags will be included by default. */
cmd_target_private_chat(id, const szArg[], iFlags = 3, bool:bContainQuotes = false) {
	new iPlayer = find_player("bl", szArg)
	if(iPlayer)
	{
		if(iPlayer != find_player("blj", szArg))
		{
			if(bContainQuotes)
			{
				client_print_green(id, _T("There are more players matching to your argument."))
			}
			else
			{
				console_print(id, _T("There are more players matching to your argument."))
			}
			return 0
		}
	}
	else if((iPlayer = find_player("c", szArg)) == 0 && (iPlayer = find_player("d", szArg)) == 0 && szArg[0] == '#' && szArg[1])
	{
		iPlayer = find_player("k", strtonum(szArg[1]))
	}

	if(!iPlayer)
	{
		if(bContainQuotes)
		{
			client_print_green(id, _T("Player with that name or userid not found."))
		}
		else
		{
			console_print(id, _T("Player with that name or userid not found."))
		}
		return 0
	}

	if(!(iFlags & 2) && iPlayer == id)
	{
		if(bContainQuotes)
		{
			client_print_green(id, _T("That action can't be performed on yourself."))
		}
		else
		{
			console_print(id, _T("That action can't be performed on yourself."))
		}
		return 0
	}

	if(iFlags & 1)
	{
		if((get_user_flags(iPlayer) & ADMIN_IMMUNITY) && (iPlayer != id) && !(get_user_flags(id) & ADMIN_SUPREME))
		{
			new szName[32]
			get_user_name(iPlayer, szName, charsmax(szName))
			if(bContainQuotes)
			{
				client_print_color2(iPlayer, id, _T("^4Player ^"^3%s^4^" has immunity."), szName)
			}
			else
			{
				console_print(id, _T("Player ^"%s^" has immunity."), szName)
			}
			return 0
		}
	}

	if(iFlags & 4)
	{
		if(!is_user_alive(iPlayer))
		{
			new szName[32]
			get_user_name(iPlayer, szName, charsmax(szName))
			if(bContainQuotes)
			{
      			client_print_color2(iPlayer, id, _T("^4That action can't be performed on dead player ^"^3%s^4^"."), szName)
			}
			else
			{
      			console_print(id, _T("That action can't be performed on dead player ^"%s^"."), szName)
			}
			return 0
		}
	}

	if(iFlags & 8)
	{
		if(is_user_bot(iPlayer))
		{
			new szName[32]
			get_user_name(iPlayer, szName, charsmax(szName))
			if(bContainQuotes)
			{
	      		client_print_color2(iPlayer, id, _T("^4That action can't be performed on bot ^"^3%s^4^"."), szName)
			}
			else
			{
	      		console_print(id, _T("That action can't be performed on bot ^"%s^"."), szName)
			}
			return 0
		}
	}
	return iPlayer
}

client_print_color2(id, iDestID, {Float,_}:...)
{
	static szMessage[192]
	szMessage[0] = 0x01

	format_args(szMessage[1], sizeof(szMessage) - 2, 2, iDestID)
	message_begin(MSG_ONE_UNRELIABLE, g_iMsgIDSayText, _, iDestID)
	write_byte((id == 0) ? 1 : id)
	write_string(szMessage)
	message_end()
}

client_print_green(id, {Float, Sql, Resul,_}:...)
{
	static szMessage[192]
	szMessage[0] = 0x04

	format_args(szMessage[1], sizeof(szMessage) - 2, 1, id)
	message_begin(MSG_ONE_UNRELIABLE, g_iMsgIDSayText, _, id)
	write_byte(id)
	write_string(szMessage)
	message_end()
}
